@@ -24,19 +24,32 @@
 # CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 # ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
-
 import hashlib
-import random
+import secrets
 import struct
-from enum import IntEnum
+from enum import Enum, IntEnum
 from typing import Any, Dict, Optional, Tuple, Union, cast
 
-from ._utils import logger_warning
-from .errors import DependencyError
+from pypdf._crypt_providers import (
+    CryptAES,
+    CryptBase,
+    CryptIdentity,
+    CryptRC4,
+    aes_cbc_decrypt,
+    aes_cbc_encrypt,
+    aes_ecb_decrypt,
+    aes_ecb_encrypt,
+    rc4_decrypt,
+    rc4_encrypt,
+)
+
+from ._utils import b_, logger_warning
 from .generic import (
     ArrayObject,
     ByteStringObject,
     DictionaryObject,
+    NameObject,
+    NumberObject,
     PdfObject,
     StreamObject,
     TextStringObject,
@@ -44,189 +57,60 @@ from .generic import (
 )
 
 
-class CryptBase:
-    def encrypt(self, data: bytes) -> bytes:  # pragma: no cover
-        return data
-
-    def decrypt(self, data: bytes) -> bytes:  # pragma: no cover
-        return data
-
-
-class CryptIdentity(CryptBase):
-    pass
-
-
-try:
-    from Crypto.Cipher import AES, ARC4  # type: ignore[import]
-    from Crypto.Util.Padding import pad  # type: ignore[import]
-
-    class CryptRC4(CryptBase):
-        def __init__(self, key: bytes) -> None:
-            self.key = key
-
-        def encrypt(self, data: bytes) -> bytes:
-            return ARC4.ARC4Cipher(self.key).encrypt(data)
-
-        def decrypt(self, data: bytes) -> bytes:
-            return ARC4.ARC4Cipher(self.key).decrypt(data)
-
-    class CryptAES(CryptBase):
-        def __init__(self, key: bytes) -> None:
-            self.key = key
-
-        def encrypt(self, data: bytes) -> bytes:
-            iv = bytes(bytearray(random.randint(0, 255) for _ in range(16)))
-            p = 16 - len(data) % 16
-            data += bytes(bytearray(p for _ in range(p)))
-            aes = AES.new(self.key, AES.MODE_CBC, iv)
-            return iv + aes.encrypt(data)
-
-        def decrypt(self, data: bytes) -> bytes:
-            iv = data[:16]
-            data = data[16:]
-            aes = AES.new(self.key, AES.MODE_CBC, iv)
-            if len(data) % 16:
-                data = pad(data, 16)
-            d = aes.decrypt(data)
-            if len(d) == 0:
-                return d
-            else:
-                return d[: -d[-1]]
-
-    def RC4_encrypt(key: bytes, data: bytes) -> bytes:
-        return ARC4.ARC4Cipher(key).encrypt(data)
-
-    def RC4_decrypt(key: bytes, data: bytes) -> bytes:
-        return ARC4.ARC4Cipher(key).decrypt(data)
-
-    def AES_ECB_encrypt(key: bytes, data: bytes) -> bytes:
-        return AES.new(key, AES.MODE_ECB).encrypt(data)
-
-    def AES_ECB_decrypt(key: bytes, data: bytes) -> bytes:
-        return AES.new(key, AES.MODE_ECB).decrypt(data)
-
-    def AES_CBC_encrypt(key: bytes, iv: bytes, data: bytes) -> bytes:
-        return AES.new(key, AES.MODE_CBC, iv).encrypt(data)
-
-    def AES_CBC_decrypt(key: bytes, iv: bytes, data: bytes) -> bytes:
-        return AES.new(key, AES.MODE_CBC, iv).decrypt(data)
-
-except ImportError:
-
-    class CryptRC4(CryptBase):  # type: ignore
-        def __init__(self, key: bytes) -> None:
-            self.S = list(range(256))
-            j = 0
-            for i in range(256):
-                j = (j + self.S[i] + key[i % len(key)]) % 256
-                self.S[i], self.S[j] = self.S[j], self.S[i]
-
-        def encrypt(self, data: bytes) -> bytes:
-            S = list(self.S)
-            out = list(0 for _ in range(len(data)))
-            i, j = 0, 0
-            for k in range(len(data)):
-                i = (i + 1) % 256
-                j = (j + S[i]) % 256
-                S[i], S[j] = S[j], S[i]
-                x = S[(S[i] + S[j]) % 256]
-                out[k] = data[k] ^ x
-            return bytes(bytearray(out))
-
-        def decrypt(self, data: bytes) -> bytes:
-            return self.encrypt(data)
-
-    class CryptAES(CryptBase):  # type: ignore
-        def __init__(self, key: bytes) -> None:
-            pass
-
-        def encrypt(self, data: bytes) -> bytes:
-            raise DependencyError("PyCryptodome is required for AES algorithm")
-
-        def decrypt(self, data: bytes) -> bytes:
-            raise DependencyError("PyCryptodome is required for AES algorithm")
-
-    def RC4_encrypt(key: bytes, data: bytes) -> bytes:
-        return CryptRC4(key).encrypt(data)
-
-    def RC4_decrypt(key: bytes, data: bytes) -> bytes:
-        return CryptRC4(key).decrypt(data)
-
-    def AES_ECB_encrypt(key: bytes, data: bytes) -> bytes:
-        raise DependencyError("PyCryptodome is required for AES algorithm")
-
-    def AES_ECB_decrypt(key: bytes, data: bytes) -> bytes:
-        raise DependencyError("PyCryptodome is required for AES algorithm")
-
-    def AES_CBC_encrypt(key: bytes, iv: bytes, data: bytes) -> bytes:
-        raise DependencyError("PyCryptodome is required for AES algorithm")
-
-    def AES_CBC_decrypt(key: bytes, iv: bytes, data: bytes) -> bytes:
-        raise DependencyError("PyCryptodome is required for AES algorithm")
-
-
 class CryptFilter:
     def __init__(
-        self, stmCrypt: CryptBase, strCrypt: CryptBase, efCrypt: CryptBase
+        self,
+        stm_crypt: CryptBase,
+        str_crypt: CryptBase,
+        ef_crypt: CryptBase,
     ) -> None:
-        self.stmCrypt = stmCrypt
-        self.strCrypt = strCrypt
-        self.efCrypt = efCrypt
+        self.stm_crypt = stm_crypt
+        self.str_crypt = str_crypt
+        self.ef_crypt = ef_crypt
 
     def encrypt_object(self, obj: PdfObject) -> PdfObject:
-        # TODO
-        return NotImplemented
+        if isinstance(obj, ByteStringObject):
+            data = self.str_crypt.encrypt(obj.original_bytes)
+            obj = ByteStringObject(data)
+        if isinstance(obj, TextStringObject):
+            data = self.str_crypt.encrypt(obj.get_encoded_bytes())
+            obj = ByteStringObject(data)
+        elif isinstance(obj, StreamObject):
+            obj2 = StreamObject()
+            obj2.update(obj)
+            obj2.set_data(self.stm_crypt.encrypt(b_(obj._data)))
+            for key, value in obj.items():  # Dont forget the Stream dict.
+                obj2[key] = self.encrypt_object(value)
+            obj = obj2
+        elif isinstance(obj, DictionaryObject):
+            obj2 = DictionaryObject()  # type: ignore
+            for key, value in obj.items():
+                obj2[key] = self.encrypt_object(value)
+            obj = obj2
+        elif isinstance(obj, ArrayObject):
+            obj = ArrayObject(self.encrypt_object(x) for x in obj)
+        return obj
 
     def decrypt_object(self, obj: PdfObject) -> PdfObject:
         if isinstance(obj, (ByteStringObject, TextStringObject)):
-            data = self.strCrypt.decrypt(obj.original_bytes)
+            data = self.str_crypt.decrypt(obj.original_bytes)
             obj = create_string_object(data)
         elif isinstance(obj, StreamObject):
-            obj._data = self.stmCrypt.decrypt(obj._data)
+            obj._data = self.stm_crypt.decrypt(b_(obj._data))
+            for key, value in obj.items():  # Dont forget the Stream dict.
+                obj[key] = self.decrypt_object(value)
         elif isinstance(obj, DictionaryObject):
-            for dictkey, value in list(obj.items()):
-                obj[dictkey] = self.decrypt_object(value)
+            for key, value in obj.items():
+                obj[key] = self.decrypt_object(value)
         elif isinstance(obj, ArrayObject):
             for i in range(len(obj)):
                 obj[i] = self.decrypt_object(obj[i])
         return obj
 
 
-_PADDING = bytes(
-    [
-        0x28,
-        0xBF,
-        0x4E,
-        0x5E,
-        0x4E,
-        0x75,
-        0x8A,
-        0x41,
-        0x64,
-        0x00,
-        0x4E,
-        0x56,
-        0xFF,
-        0xFA,
-        0x01,
-        0x08,
-        0x2E,
-        0x2E,
-        0x00,
-        0xB6,
-        0xD0,
-        0x68,
-        0x3E,
-        0x80,
-        0x2F,
-        0x0C,
-        0xA9,
-        0xFE,
-        0x64,
-        0x53,
-        0x69,
-        0x7A,
-    ]
+_PADDING = (
+    b"\x28\xbf\x4e\x5e\x4e\x75\x8a\x41\x64\x00\x4e\x56\xff\xfa\x01\x08"
+    b"\x2e\x2e\x00\xb6\xd0\x68\x3e\x80\x2f\x0c\xa9\xfe\x64\x53\x69\x7a"
 )
 
 
@@ -310,7 +194,7 @@ class AlgV4:
         u_hash.update(o_entry)
         u_hash.update(struct.pack("<I", P))
         u_hash.update(id1_entry)
-        if rev >= 4 and metadata_encrypted is False:
+        if rev >= 4 and not metadata_encrypted:
             u_hash.update(b"\xff\xff\xff\xff")
         u_hash_digest = u_hash.digest()
         length = key_size // 8
@@ -384,11 +268,11 @@ class AlgV4:
             The RC4 encrypted
         """
         a = _padding(user_password)
-        rc4_enc = RC4_encrypt(rc4_key, a)
+        rc4_enc = rc4_encrypt(rc4_key, a)
         if rev >= 3:
             for i in range(1, 20):
                 key = bytes(bytearray(x ^ i for x in rc4_key))
-                rc4_enc = RC4_encrypt(key, rc4_enc)
+                rc4_enc = rc4_encrypt(key, rc4_enc)
         return rc4_enc
 
     @staticmethod
@@ -415,7 +299,7 @@ class AlgV4:
             The value
         """
         if rev <= 2:
-            value = RC4_encrypt(key, _PADDING)
+            value = rc4_encrypt(key, _PADDING)
             return value
 
         """
@@ -445,10 +329,10 @@ class AlgV4:
         """
         u_hash = hashlib.md5(_PADDING)
         u_hash.update(id1_entry)
-        rc4_enc = RC4_encrypt(key, u_hash.digest())
+        rc4_enc = rc4_encrypt(key, u_hash.digest())
         for i in range(1, 20):
             rc4_key = bytes(bytearray(x ^ i for x in key))
-            rc4_enc = RC4_encrypt(rc4_key, rc4_enc)
+            rc4_enc = rc4_encrypt(rc4_key, rc4_enc)
         return _padding(rc4_enc)
 
     @staticmethod
@@ -482,7 +366,7 @@ class AlgV4:
            to decrypt the document.
 
         Args:
-            user_password: The user passwort as a bytes stream
+            user_password: The user password as a bytes stream
             rev: The encryption revision (see PDF standard)
             key_size: The size of the key in bytes
             o_entry: The owner entry
@@ -563,12 +447,12 @@ class AlgV4:
         rc4_key = AlgV4.compute_O_value_key(owner_password, rev, key_size)
 
         if rev <= 2:
-            user_password = RC4_decrypt(rc4_key, o_entry)
+            user_password = rc4_decrypt(rc4_key, o_entry)
         else:
             user_password = o_entry
             for i in range(19, -1, -1):
                 key = bytes(bytearray(x ^ i for x in rc4_key))
-                user_password = RC4_decrypt(key, user_password)
+                user_password = rc4_decrypt(key, user_password)
         return AlgV4.verify_user_password(
             user_password,
             rev,
@@ -651,7 +535,7 @@ class AlgV5:
             return b""
         iv = bytes(0 for _ in range(16))
         tmp_key = AlgV5.calculate_hash(R, password, o_value[40:48], u_value[:48])
-        key = AES_CBC_decrypt(tmp_key, iv, oe_value)
+        key = aes_cbc_decrypt(tmp_key, iv, oe_value)
         return key
 
     @staticmethod
@@ -678,28 +562,28 @@ class AlgV5:
             return b""
         iv = bytes(0 for _ in range(16))
         tmp_key = AlgV5.calculate_hash(R, password, u_value[40:48], b"")
-        return AES_CBC_decrypt(tmp_key, iv, ue_value)
+        return aes_cbc_decrypt(tmp_key, iv, ue_value)
 
     @staticmethod
     def calculate_hash(R: int, password: bytes, salt: bytes, udata: bytes) -> bytes:
         # from https://github.com/qpdf/qpdf/blob/main/libqpdf/QPDF_encryption.cc
-        K = hashlib.sha256(password + salt + udata).digest()
+        k = hashlib.sha256(password + salt + udata).digest()
         if R < 6:
-            return K
+            return k
         count = 0
         while True:
             count += 1
-            K1 = password + K + udata
-            E = AES_CBC_encrypt(K[:16], K[16:32], K1 * 64)
+            k1 = password + k + udata
+            e = aes_cbc_encrypt(k[:16], k[16:32], k1 * 64)
             hash_fn = (
                 hashlib.sha256,
                 hashlib.sha384,
                 hashlib.sha512,
-            )[sum(E[:16]) % 3]
-            K = hash_fn(E).digest()
-            if count >= 64 and E[-1] <= count - 32:
+            )[sum(e[:16]) % 3]
+            k = hash_fn(e).digest()
+            if count >= 64 and e[-1] <= count - 32:
                 break
-        return K[:32]
+        return k[:32]
 
     @staticmethod
     def verify_perms(
@@ -724,19 +608,22 @@ class AlgV5:
         """
         b8 = b"T" if metadata_encrypted else b"F"
         p1 = struct.pack("<I", p) + b"\xff\xff\xff\xff" + b8 + b"adb"
-        p2 = AES_ECB_decrypt(key, perms)
+        p2 = aes_ecb_decrypt(key, perms)
         return p1 == p2[:12]
 
     @staticmethod
     def generate_values(
+        R: int,
         user_password: bytes,
         owner_password: bytes,
         key: bytes,
         p: int,
         metadata_encrypted: bool,
     ) -> Dict[Any, Any]:
-        u_value, ue_value = AlgV5.compute_U_value(user_password, key)
-        o_value, oe_value = AlgV5.compute_O_value(owner_password, key, u_value)
+        user_password = user_password[:127]
+        owner_password = owner_password[:127]
+        u_value, ue_value = AlgV5.compute_U_value(R, user_password, key)
+        o_value, oe_value = AlgV5.compute_O_value(R, owner_password, key, u_value)
         perms = AlgV5.compute_Perms_value(key, p, metadata_encrypted)
         return {
             "/U": u_value,
@@ -747,7 +634,7 @@ class AlgV5:
         }
 
     @staticmethod
-    def compute_U_value(password: bytes, key: bytes) -> Tuple[bytes, bytes]:
+    def compute_U_value(R: int, password: bytes, key: bytes) -> Tuple[bytes, bytes]:
         """
         Algorithm 3.8 Computing the encryption dictionary’s U (user password)
         and UE (user encryption key) values.
@@ -765,25 +652,26 @@ class AlgV5:
            as the UE key.
 
         Args:
+            R:
             password:
             key:
 
         Returns:
             A tuple (u-value, ue value)
         """
-        random_bytes = bytes(random.randrange(0, 256) for _ in range(16))
+        random_bytes = secrets.token_bytes(16)
         val_salt = random_bytes[:8]
         key_salt = random_bytes[8:]
-        u_value = hashlib.sha256(password + val_salt).digest() + val_salt + key_salt
+        u_value = AlgV5.calculate_hash(R, password, val_salt, b"") + val_salt + key_salt
 
-        tmp_key = hashlib.sha256(password + key_salt).digest()
+        tmp_key = AlgV5.calculate_hash(R, password, key_salt, b"")
         iv = bytes(0 for _ in range(16))
-        ue_value = AES_CBC_encrypt(tmp_key, iv, key)
+        ue_value = aes_cbc_encrypt(tmp_key, iv, key)
         return u_value, ue_value
 
     @staticmethod
     def compute_O_value(
-        password: bytes, key: bytes, u_value: bytes
+        R: int, password: bytes, key: bytes, u_value: bytes
     ) -> Tuple[bytes, bytes]:
         """
         Algorithm 3.9 Computing the encryption dictionary’s O (owner password)
@@ -805,6 +693,7 @@ class AlgV5:
            The resulting 32-byte string is stored as the OE key.
 
         Args:
+            R:
             password:
             key:
             u_value: A 32-byte string, based on the user password, that shall be
@@ -814,16 +703,15 @@ class AlgV5:
         Returns:
             A tuple (O value, OE value)
         """
-        random_bytes = bytes(random.randrange(0, 256) for _ in range(16))
+        random_bytes = secrets.token_bytes(16)
         val_salt = random_bytes[:8]
         key_salt = random_bytes[8:]
         o_value = (
-            hashlib.sha256(password + val_salt + u_value).digest() + val_salt + key_salt
+            AlgV5.calculate_hash(R, password, val_salt, u_value) + val_salt + key_salt
         )
-
-        tmp_key = hashlib.sha256(password + key_salt + u_value).digest()
+        tmp_key = AlgV5.calculate_hash(R, password, key_salt, u_value[:48])
         iv = bytes(0 for _ in range(16))
-        oe_value = AES_CBC_encrypt(tmp_key, iv, key)
+        oe_value = aes_cbc_encrypt(tmp_key, iv, key)
         return o_value, oe_value
 
     @staticmethod
@@ -859,9 +747,9 @@ class AlgV5:
             The perms value
         """
         b8 = b"T" if metadata_encrypted else b"F"
-        rr = bytes(random.randrange(0, 256) for _ in range(4))
+        rr = secrets.token_bytes(4)
         data = struct.pack("<I", p) + b"\xff\xff\xff\xff" + b8 + b"adb" + rr
-        perms = AES_ECB_encrypt(key, data)
+        perms = aes_ecb_encrypt(key, data)
         return perms
 
 
@@ -871,36 +759,110 @@ class PasswordType(IntEnum):
     OWNER_PASSWORD = 2
 
 
+class EncryptAlgorithm(tuple, Enum):  # type: ignore # noqa: SLOT001
+    # V, R, Length
+    RC4_40 = (1, 2, 40)
+    RC4_128 = (2, 3, 128)
+    AES_128 = (4, 4, 128)
+    AES_256_R5 = (5, 5, 256)
+    AES_256 = (5, 6, 256)
+
+
+class EncryptionValues:
+    O: bytes  # noqa
+    U: bytes
+    OE: bytes
+    UE: bytes
+    Perms: bytes
+
+
 class Encryption:
+    """
+    Collects and manages parameters for PDF document encryption and decryption.
+
+    Args:
+        V: A code specifying the algorithm to be used in encrypting and
+           decrypting the document.
+        R: The revision of the standard security handler.
+        Length: The length of the encryption key in bits.
+        P: A set of flags specifying which operations shall be permitted
+           when the document is opened with user access
+        entry: The encryption dictionary object.
+        EncryptMetadata: Whether to encrypt metadata in the document.
+        first_id_entry: The first 16 bytes of the file's original ID.
+        StmF: The name of the crypt filter that shall be used by default
+              when decrypting streams.
+        StrF: The name of the crypt filter that shall be used when decrypting
+              all strings in the document.
+        EFF: The name of the crypt filter that shall be used when
+             encrypting embedded file streams that do not have their own
+             crypt filter specifier.
+        values: Additional encryption parameters.
+    """
+
     def __init__(
         self,
-        algV: int,
-        algR: int,
+        V: int,
+        R: int,
+        Length: int,
+        P: int,
         entry: DictionaryObject,
+        EncryptMetadata: bool,
         first_id_entry: bytes,
         StmF: str,
         StrF: str,
         EFF: str,
+        values: Optional[EncryptionValues],
     ) -> None:
-        # See TABLE 3.18 Entries common to all encryption dictionaries
-        self.algV = algV
-        self.algR = algR
-        self.entry = entry
-        self.key_size = entry.get("/Length", 40)
+        # §7.6.2, entries common to all encryption dictionaries
+        # use same name as keys of encryption dictionaries entries
+        self.V = V
+        self.R = R
+        self.Length = Length  # key_size
+        self.P = (P + 0x100000000) % 0x100000000  # maybe P < 0
+        self.EncryptMetadata = EncryptMetadata
         self.id1_entry = first_id_entry
         self.StmF = StmF
         self.StrF = StrF
         self.EFF = EFF
+        self.values: EncryptionValues = values if values else EncryptionValues()
 
-        # 1 => owner password
-        # 2 => user password
         self._password_type = PasswordType.NOT_DECRYPTED
         self._key: Optional[bytes] = None
 
     def is_decrypted(self) -> bool:
         return self._password_type != PasswordType.NOT_DECRYPTED
 
+    def encrypt_object(self, obj: PdfObject, idnum: int, generation: int) -> PdfObject:
+        # skip calculate key
+        if not self._is_encryption_object(obj):
+            return obj
+
+        cf = self._make_crypt_filter(idnum, generation)
+        return cf.encrypt_object(obj)
+
     def decrypt_object(self, obj: PdfObject, idnum: int, generation: int) -> PdfObject:
+        # skip calculate key
+        if not self._is_encryption_object(obj):
+            return obj
+
+        cf = self._make_crypt_filter(idnum, generation)
+        return cf.decrypt_object(obj)
+
+    @staticmethod
+    def _is_encryption_object(obj: PdfObject) -> bool:
+        return isinstance(
+            obj,
+            (
+                ByteStringObject,
+                TextStringObject,
+                StreamObject,
+                ArrayObject,
+                DictionaryObject,
+            ),
+        )
+
+    def _make_crypt_filter(self, idnum: int, generation: int) -> CryptFilter:
         """
         Algorithm 1: Encryption of data using the RC4 or AES algorithms.
 
@@ -939,21 +901,13 @@ class Encryption:
            16 bytes, and the initialization vector is a 16-byte random number
            that is stored as the first 16 bytes of the encrypted stream or string.
            The output is the encrypted data to be stored in the PDF file.
-
-        Args:
-            obj:
-            idnum:
-            generation:
-
-        Returns:
-            The PdfObject
         """
         pack1 = struct.pack("<i", idnum)[:3]
         pack2 = struct.pack("<i", generation)[:2]
 
         assert self._key
         key = self._key
-        n = 5 if self.algV == 1 else self.key_size // 8
+        n = 5 if self.V == 1 else self.Length // 8
         key_data = key[:n] + pack1 + pack2
         key_hash = hashlib.md5(key_data)
         rc4_key = key_hash.digest()[: min(n + 5, 16)]
@@ -964,12 +918,11 @@ class Encryption:
         # for AES-256
         aes256_key = key
 
-        stmCrypt = self._get_crypt(self.StmF, rc4_key, aes128_key, aes256_key)
-        StrCrypt = self._get_crypt(self.StrF, rc4_key, aes128_key, aes256_key)
-        efCrypt = self._get_crypt(self.EFF, rc4_key, aes128_key, aes256_key)
+        stm_crypt = self._get_crypt(self.StmF, rc4_key, aes128_key, aes256_key)
+        str_crypt = self._get_crypt(self.StrF, rc4_key, aes128_key, aes256_key)
+        ef_crypt = self._get_crypt(self.EFF, rc4_key, aes128_key, aes256_key)
 
-        cf = CryptFilter(stmCrypt, StrCrypt, efCrypt)
-        return cf.decrypt_object(obj)
+        return CryptFilter(stm_crypt, str_crypt, ef_crypt)
 
     @staticmethod
     def _get_crypt(
@@ -984,53 +937,48 @@ class Encryption:
         else:
             return CryptRC4(rc4_key)
 
-    def verify(self, password: Union[bytes, str]) -> PasswordType:
+    @staticmethod
+    def _encode_password(password: Union[bytes, str]) -> bytes:
         if isinstance(password, str):
             try:
                 pwd = password.encode("latin-1")
-            except Exception:  # noqa
+            except Exception:
                 pwd = password.encode("utf-8")
         else:
             pwd = password
+        return pwd
 
-        key, rc = self.verify_v4(pwd) if self.algV <= 4 else self.verify_v5(pwd)
+    def verify(self, password: Union[bytes, str]) -> PasswordType:
+        pwd = self._encode_password(password)
+        key, rc = self.verify_v4(pwd) if self.V <= 4 else self.verify_v5(pwd)
         if rc != PasswordType.NOT_DECRYPTED:
             self._password_type = rc
             self._key = key
         return rc
 
     def verify_v4(self, password: bytes) -> Tuple[bytes, PasswordType]:
-        R = cast(int, self.entry["/R"])
-        P = cast(int, self.entry["/P"])
-        P = (P + 0x100000000) % 0x100000000  # maybe < 0
-        # make type(metadata_encrypted) == bool
-        em = self.entry.get("/EncryptMetadata")
-        metadata_encrypted = em.value if em else True
-        o_entry = cast(ByteStringObject, self.entry["/O"].get_object()).original_bytes
-        u_entry = cast(ByteStringObject, self.entry["/U"].get_object()).original_bytes
-
         # verify owner password first
         key = AlgV4.verify_owner_password(
             password,
-            R,
-            self.key_size,
-            o_entry,
-            u_entry,
-            P,
+            self.R,
+            self.Length,
+            self.values.O,
+            self.values.U,
+            self.P,
             self.id1_entry,
-            metadata_encrypted,
+            self.EncryptMetadata,
         )
         if key:
             return key, PasswordType.OWNER_PASSWORD
         key = AlgV4.verify_user_password(
             password,
-            R,
-            self.key_size,
-            o_entry,
-            u_entry,
-            P,
+            self.R,
+            self.Length,
+            self.values.O,
+            self.values.U,
+            self.P,
             self.id1_entry,
-            metadata_encrypted,
+            self.EncryptMetadata,
         )
         if key:
             return key, PasswordType.USER_PASSWORD
@@ -1038,69 +986,183 @@ class Encryption:
 
     def verify_v5(self, password: bytes) -> Tuple[bytes, PasswordType]:
         # TODO: use SASLprep process
-        o_entry = cast(ByteStringObject, self.entry["/O"].get_object()).original_bytes
-        u_entry = cast(ByteStringObject, self.entry["/U"].get_object()).original_bytes
-        oe_entry = cast(ByteStringObject, self.entry["/OE"].get_object()).original_bytes
-        ue_entry = cast(ByteStringObject, self.entry["/UE"].get_object()).original_bytes
-
         # verify owner password first
         key = AlgV5.verify_owner_password(
-            self.algR, password, o_entry, oe_entry, u_entry
+            self.R, password, self.values.O, self.values.OE, self.values.U
         )
         rc = PasswordType.OWNER_PASSWORD
         if not key:
-            key = AlgV5.verify_user_password(self.algR, password, u_entry, ue_entry)
+            key = AlgV5.verify_user_password(
+                self.R, password, self.values.U, self.values.UE
+            )
             rc = PasswordType.USER_PASSWORD
         if not key:
             return b"", PasswordType.NOT_DECRYPTED
 
         # verify Perms
-        perms = cast(ByteStringObject, self.entry["/Perms"].get_object()).original_bytes
-        P = cast(int, self.entry["/P"])
-        P = (P + 0x100000000) % 0x100000000  # maybe < 0
-        metadata_encrypted = self.entry.get("/EncryptMetadata", True)
-        if not AlgV5.verify_perms(key, perms, P, metadata_encrypted):
+        if not AlgV5.verify_perms(key, self.values.Perms, self.P, self.EncryptMetadata):
             logger_warning("ignore '/Perms' verify failed", __name__)
         return key, rc
 
+    def write_entry(
+        self, user_password: str, owner_password: Optional[str]
+    ) -> DictionaryObject:
+        user_pwd = self._encode_password(user_password)
+        owner_pwd = self._encode_password(owner_password) if owner_password else None
+        if owner_pwd is None:
+            owner_pwd = user_pwd
+
+        if self.V <= 4:
+            self.compute_values_v4(user_pwd, owner_pwd)
+        else:
+            self._key = secrets.token_bytes(self.Length // 8)
+            values = AlgV5.generate_values(
+                self.R, user_pwd, owner_pwd, self._key, self.P, self.EncryptMetadata
+            )
+            self.values.O = values["/O"]
+            self.values.U = values["/U"]
+            self.values.OE = values["/OE"]
+            self.values.UE = values["/UE"]
+            self.values.Perms = values["/Perms"]
+
+        dict_obj = DictionaryObject()
+        dict_obj[NameObject("/V")] = NumberObject(self.V)
+        dict_obj[NameObject("/R")] = NumberObject(self.R)
+        dict_obj[NameObject("/Length")] = NumberObject(self.Length)
+        dict_obj[NameObject("/P")] = NumberObject(self.P)
+        dict_obj[NameObject("/Filter")] = NameObject("/Standard")
+        # ignore /EncryptMetadata
+
+        dict_obj[NameObject("/O")] = ByteStringObject(self.values.O)
+        dict_obj[NameObject("/U")] = ByteStringObject(self.values.U)
+
+        if self.V >= 4:
+            # TODO: allow different method
+            std_cf = DictionaryObject()
+            std_cf[NameObject("/AuthEvent")] = NameObject("/DocOpen")
+            std_cf[NameObject("/CFM")] = NameObject(self.StmF)
+            std_cf[NameObject("/Length")] = NumberObject(self.Length // 8)
+            cf = DictionaryObject()
+            cf[NameObject("/StdCF")] = std_cf
+            dict_obj[NameObject("/CF")] = cf
+            dict_obj[NameObject("/StmF")] = NameObject("/StdCF")
+            dict_obj[NameObject("/StrF")] = NameObject("/StdCF")
+            # ignore EFF
+            # dict_obj[NameObject("/EFF")] = NameObject("/StdCF")
+
+        if self.V >= 5:
+            dict_obj[NameObject("/OE")] = ByteStringObject(self.values.OE)
+            dict_obj[NameObject("/UE")] = ByteStringObject(self.values.UE)
+            dict_obj[NameObject("/Perms")] = ByteStringObject(self.values.Perms)
+        return dict_obj
+
+    def compute_values_v4(self, user_password: bytes, owner_password: bytes) -> None:
+        rc4_key = AlgV4.compute_O_value_key(owner_password, self.R, self.Length)
+        o_value = AlgV4.compute_O_value(rc4_key, user_password, self.R)
+
+        key = AlgV4.compute_key(
+            user_password,
+            self.R,
+            self.Length,
+            o_value,
+            self.P,
+            self.id1_entry,
+            self.EncryptMetadata,
+        )
+        u_value = AlgV4.compute_U_value(key, self.R, self.id1_entry)
+
+        self._key = key
+        self.values.O = o_value
+        self.values.U = u_value
+
     @staticmethod
     def read(encryption_entry: DictionaryObject, first_id_entry: bytes) -> "Encryption":
-        filter = encryption_entry.get("/Filter")
-        if filter != "/Standard":
+        if encryption_entry.get("/Filter") != "/Standard":
             raise NotImplementedError(
                 "only Standard PDF encryption handler is available"
             )
         if "/SubFilter" in encryption_entry:
             raise NotImplementedError("/SubFilter NOT supported")
 
-        StmF = "/V2"
-        StrF = "/V2"
-        EFF = "/V2"
+        stm_filter = "/V2"
+        str_filter = "/V2"
+        ef_filter = "/V2"
 
-        V = encryption_entry.get("/V", 0)
-        if V not in (1, 2, 3, 4, 5):
-            raise NotImplementedError(f"Encryption V={V} NOT supported")
-        if V >= 4:
+        alg_ver = encryption_entry.get("/V", 0)
+        if alg_ver not in (1, 2, 3, 4, 5):
+            raise NotImplementedError(f"Encryption V={alg_ver} NOT supported")
+        if alg_ver >= 4:
             filters = encryption_entry["/CF"]
 
-            StmF = encryption_entry.get("/StmF", "/Identity")
-            StrF = encryption_entry.get("/StrF", "/Identity")
-            EFF = encryption_entry.get("/EFF", StmF)
+            stm_filter = encryption_entry.get("/StmF", "/Identity")
+            str_filter = encryption_entry.get("/StrF", "/Identity")
+            ef_filter = encryption_entry.get("/EFF", stm_filter)
 
-            if StmF != "/Identity":
-                StmF = filters[StmF]["/CFM"]  # type: ignore
-            if StrF != "/Identity":
-                StrF = filters[StrF]["/CFM"]  # type: ignore
-            if EFF != "/Identity":
-                EFF = filters[EFF]["/CFM"]  # type: ignore
+            if stm_filter != "/Identity":
+                stm_filter = filters[stm_filter]["/CFM"]  # type: ignore
+            if str_filter != "/Identity":
+                str_filter = filters[str_filter]["/CFM"]  # type: ignore
+            if ef_filter != "/Identity":
+                ef_filter = filters[ef_filter]["/CFM"]  # type: ignore
 
             allowed_methods = ("/Identity", "/V2", "/AESV2", "/AESV3")
-            if StmF not in allowed_methods:
-                raise NotImplementedError("StmF Method {StmF} NOT supported!")
-            if StrF not in allowed_methods:
-                raise NotImplementedError(f"StrF Method {StrF} NOT supported!")
-            if EFF not in allowed_methods:
-                raise NotImplementedError(f"EFF Method {EFF} NOT supported!")
+            if stm_filter not in allowed_methods:
+                raise NotImplementedError(f"StmF Method {stm_filter} NOT supported!")
+            if str_filter not in allowed_methods:
+                raise NotImplementedError(f"StrF Method {str_filter} NOT supported!")
+            if ef_filter not in allowed_methods:
+                raise NotImplementedError(f"EFF Method {ef_filter} NOT supported!")
 
-        R = cast(int, encryption_entry["/R"])
-        return Encryption(V, R, encryption_entry, first_id_entry, StmF, StrF, EFF)
+        alg_rev = cast(int, encryption_entry["/R"])
+        perm_flags = cast(int, encryption_entry["/P"])
+        key_bits = encryption_entry.get("/Length", 40)
+        encrypt_metadata = encryption_entry.get("/EncryptMetadata")
+        encrypt_metadata = (
+            encrypt_metadata.value if encrypt_metadata is not None else True
+        )
+        values = EncryptionValues()
+        values.O = cast(ByteStringObject, encryption_entry["/O"]).original_bytes
+        values.U = cast(ByteStringObject, encryption_entry["/U"]).original_bytes
+        values.OE = encryption_entry.get("/OE", ByteStringObject()).original_bytes
+        values.UE = encryption_entry.get("/UE", ByteStringObject()).original_bytes
+        values.Perms = encryption_entry.get("/Perms", ByteStringObject()).original_bytes
+        return Encryption(
+            V=alg_ver,
+            R=alg_rev,
+            Length=key_bits,
+            P=perm_flags,
+            EncryptMetadata=encrypt_metadata,
+            first_id_entry=first_id_entry,
+            values=values,
+            StrF=str_filter,
+            StmF=stm_filter,
+            EFF=ef_filter,
+            entry=encryption_entry,  # Dummy entry for the moment; will get removed
+        )
+
+    @staticmethod
+    def make(
+        alg: EncryptAlgorithm, permissions: int, first_id_entry: bytes
+    ) -> "Encryption":
+        alg_ver, alg_rev, key_bits = alg
+
+        stm_filter, str_filter, ef_filter = "/V2", "/V2", "/V2"
+
+        if alg == EncryptAlgorithm.AES_128:
+            stm_filter, str_filter, ef_filter = "/AESV2", "/AESV2", "/AESV2"
+        elif alg in (EncryptAlgorithm.AES_256_R5, EncryptAlgorithm.AES_256):
+            stm_filter, str_filter, ef_filter = "/AESV3", "/AESV3", "/AESV3"
+
+        return Encryption(
+            V=alg_ver,
+            R=alg_rev,
+            Length=key_bits,
+            P=permissions,
+            EncryptMetadata=True,
+            first_id_entry=first_id_entry,
+            values=None,
+            StrF=str_filter,
+            StmF=stm_filter,
+            EFF=ef_filter,
+            entry=DictionaryObject(),  # Dummy entry for the moment; will get removed
+        )
